#!/usr/local/bin/luac53 -p --
--[[
--@brief  Quasi Robot Language; LuaChip
--@date   Sat,12 May,2018
--@date   Sat,02 Jun,2018
--@date   Sun,02 Dec,2018
--@date   Tue,01 Jan,2019
--@date   Sun,20 Jan,2019
--@date   Fri,01 Feb,2019
--@date   Sat,16 Feb,2019
--@date   Tue,28 Apr,2020
--@date   Sun,13 Sep,2020
--@date   Sat,19 Sep,2020
--@date   Sat,30 Oct,2021
--@date   Sun,06 Mar,2022
--@date   Mon,09 Jan,2023
--@date   Mon,26 Jun,2023
--@date   Tue,27 Jun,2023
--@author Copyright(C)2018-2023 G-HAL
--]]

-- ##begin##
-- name : Get Direction of the Mek
-- size : 6 , 1
-- footprint :
-- ######
-- address :
-- chip_coordinates_tag : ##010##
-- chip_number_tag : ##011##
-- chip_number : 
-- chip_footprint_tag : ##012##
-- next_address_condition_true_coordinates_tag : ##020##
-- next_address_condition_true_tag : ##021##
-- next_address_condition_true : 0010013
-- argument : 2
-- argument_comment[1] : Direction
-- argument[1] : var
-- argument_tag[1] : ##111##
-- argument_type[1] : var
-- argument_geometry[1] : x:16 , y:16 , w:123 , h:31
-- argument_comment[2] : Mek
-- argument[2] : mek
-- argument_tag[2] : ##121##
-- argument_type[2] : var
-- argument_geometry[2] : x:278 , y:16 , w:123 , h:31
-- ##body##

function qrl_chip##011##(Mek, GB)
  PrintTraceMessage(GB, Mek, '##010##:##011##:Get_Direction_of_the_Mek');
  PrintTracer(GB, Mek, '##012##');
  if enable_debugger then
    dbg();
  end;
  if (c_Variable_Type_Mek ~= g_QRL_UserVariable_Type["##121##"]) then
    PrintErrorMessage(GB, Mek, '##010##:##011##:Get_Direction_of_the_Mek:No mek is selected.');
    return '##021##';
  end;

  local t_x, t_y, t_z = GearCurrentLocation(g_QRL_UserVariable_Contents["##121##"]);
  local m_x, m_y, m_z = GearCurrentLocation(Mek);
  local x_rel = (t_x - m_x);
  local y_rel = (t_y - m_y);
  local z_rel = (t_z - m_z);
  local current_dir = Mek_NAttValue_Location(Mek, NAS_D);
  g_QRL_UserVariable_Type["##111##"] = c_Variable_Type_Num;
  g_QRL_UserVariable_Contents["##111##"] = relpos2dir(x_rel, y_rel, z_rel, current_dir);
  return '##021##';
end;

-- ##end##
-- [ End of File ]
